@@ -1,5 +1,5 @@
 /* BFD support for the ARM processor
-   Copyright (C) 1994-2018 Free Software Foundation, Inc.
+   Copyright (C) 1994-2019 Free Software Foundation, Inc.
    Contributed by Richard Earnshaw (rwe@pegasus.esprit.ec.org)
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -65,36 +65,129 @@ static struct
 }
 processors[] =
 {
-  { bfd_mach_arm_2,  "arm2"     },
-  { bfd_mach_arm_2a, "arm250"   },
-  { bfd_mach_arm_2a, "arm3"     },
-  { bfd_mach_arm_3,  "arm6"     },
-  { bfd_mach_arm_3,  "arm60"    },
-  { bfd_mach_arm_3,  "arm600"   },
-  { bfd_mach_arm_3,  "arm610"   },
-  { bfd_mach_arm_3,  "arm7"     },
-  { bfd_mach_arm_3,  "arm710"   },
-  { bfd_mach_arm_3,  "arm7500"  },
-  { bfd_mach_arm_3,  "arm7d"    },
-  { bfd_mach_arm_3,  "arm7di"   },
-  { bfd_mach_arm_3M, "arm7dm"   },
-  { bfd_mach_arm_3M, "arm7dmi"  },
-  { bfd_mach_arm_4T, "arm7tdmi" },
-  { bfd_mach_arm_4,  "arm8"     },
-  { bfd_mach_arm_4,  "arm810"   },
-  { bfd_mach_arm_4,  "arm9"     },
-  { bfd_mach_arm_4,  "arm920"   },
-  { bfd_mach_arm_4T, "arm920t"  },
-  { bfd_mach_arm_4T, "arm9tdmi" },
-  { bfd_mach_arm_4,  "sa1"      },
-  { bfd_mach_arm_4,  "strongarm"},
-  { bfd_mach_arm_4,  "strongarm110" },
-  { bfd_mach_arm_4,  "strongarm1100" },
-  { bfd_mach_arm_XScale, "xscale" },
-  { bfd_mach_arm_ep9312, "ep9312" },
-  { bfd_mach_arm_iWMMXt, "iwmmxt" },
-  { bfd_mach_arm_iWMMXt2, "iwmmxt2" },
-  { bfd_mach_arm_unknown, "arm_any" }
+  { bfd_mach_arm_2,	  "arm2"	    },
+  { bfd_mach_arm_2a,	  "arm250"	    },
+  { bfd_mach_arm_2a,	  "arm3"	    },
+  { bfd_mach_arm_3,	  "arm6"	    },
+  { bfd_mach_arm_3,	  "arm60"	    },
+  { bfd_mach_arm_3,	  "arm600"	    },
+  { bfd_mach_arm_3,	  "arm610"	    },
+  { bfd_mach_arm_3,	  "arm620"	    },
+  { bfd_mach_arm_3,	  "arm7"	    },
+  { bfd_mach_arm_3,	  "arm70"	    },
+  { bfd_mach_arm_3,	  "arm700"	    },
+  { bfd_mach_arm_3,	  "arm700i"	    },
+  { bfd_mach_arm_3,	  "arm710"	    },
+  { bfd_mach_arm_3,	  "arm7100"	    },
+  { bfd_mach_arm_3,	  "arm710c"	    },
+  { bfd_mach_arm_4T,	  "arm710t"	    },
+  { bfd_mach_arm_3,	  "arm720"	    },
+  { bfd_mach_arm_4T,	  "arm720t"	    },
+  { bfd_mach_arm_4T,	  "arm740t"	    },
+  { bfd_mach_arm_3,	  "arm7500"	    },
+  { bfd_mach_arm_3,	  "arm7500fe"	    },
+  { bfd_mach_arm_3,	  "arm7d"	    },
+  { bfd_mach_arm_3,	  "arm7di"	    },
+  { bfd_mach_arm_3M,	  "arm7dm"	    },
+  { bfd_mach_arm_3M,	  "arm7dmi"	    },
+  { bfd_mach_arm_4T,	  "arm7t"	    },
+  { bfd_mach_arm_4T,	  "arm7tdmi"	    },
+  { bfd_mach_arm_4T,	  "arm7tdmi-s"	    },
+  { bfd_mach_arm_3M,	  "arm7m"	    },
+  { bfd_mach_arm_4,	  "arm8"	    },
+  { bfd_mach_arm_4,	  "arm810"	    },
+  { bfd_mach_arm_4,	  "arm9"	    },
+  { bfd_mach_arm_4T,	  "arm920"	    },
+  { bfd_mach_arm_4T,	  "arm920t"	    },
+  { bfd_mach_arm_4T,	  "arm922t"	    },
+  { bfd_mach_arm_5TEJ,	  "arm926ej"	    },
+  { bfd_mach_arm_5TEJ,	  "arm926ejs"	    },
+  { bfd_mach_arm_5TEJ,	  "arm926ej-s"	    },
+  { bfd_mach_arm_4T,	  "arm940t"	    },
+  { bfd_mach_arm_5TE,	  "arm946e"	    },
+  { bfd_mach_arm_5TE,	  "arm946e-r0"	    },
+  { bfd_mach_arm_5TE,	  "arm946e-s"	    },
+  { bfd_mach_arm_5TE,	  "arm966e"	    },
+  { bfd_mach_arm_5TE,	  "arm966e-r0"	    },
+  { bfd_mach_arm_5TE,	  "arm966e-s"	    },
+  { bfd_mach_arm_5TE,	  "arm968e-s"	    },
+  { bfd_mach_arm_5TE,	  "arm9e"	    },
+  { bfd_mach_arm_5TE,	  "arm9e-r0"	    },
+  { bfd_mach_arm_4T,	  "arm9tdmi"	    },
+  { bfd_mach_arm_5TE,	  "arm1020"	    },
+  { bfd_mach_arm_5T,	  "arm1020t"	    },
+  { bfd_mach_arm_5TE,	  "arm1020e"	    },
+  { bfd_mach_arm_5TE,	  "arm1022e"	    },
+  { bfd_mach_arm_5TEJ,	  "arm1026ejs"	    },
+  { bfd_mach_arm_5TEJ,	  "arm1026ej-s"	    },
+  { bfd_mach_arm_5TE,	  "arm10e"	    },
+  { bfd_mach_arm_5T,	  "arm10t"	    },
+  { bfd_mach_arm_5T,	  "arm10tdmi"	    },
+  { bfd_mach_arm_6,	  "arm1136j-s"	    },
+  { bfd_mach_arm_6,	  "arm1136js"	    },
+  { bfd_mach_arm_6,	  "arm1136jf-s"	    },
+  { bfd_mach_arm_6,	  "arm1136jfs"	    },
+  { bfd_mach_arm_6KZ,	  "arm1176jz-s"	    },
+  { bfd_mach_arm_6KZ,	  "arm1176jzf-s"    },
+  { bfd_mach_arm_6T2,	  "arm1156t2-s"	    },
+  { bfd_mach_arm_6T2,	  "arm1156t2f-s"    },
+  { bfd_mach_arm_7,	  "cortex-a5"	    },
+  { bfd_mach_arm_7,	  "cortex-a7"	    },
+  { bfd_mach_arm_7,	  "cortex-a8"	    },
+  { bfd_mach_arm_7,	  "cortex-a9"	    },
+  { bfd_mach_arm_7,	  "cortex-a12"	    },
+  { bfd_mach_arm_7,	  "cortex-a15"	    },
+  { bfd_mach_arm_7,	  "cortex-a17"	    },
+  { bfd_mach_arm_8,	  "cortex-a32"	    },
+  { bfd_mach_arm_8,	  "cortex-a35"	    },
+  { bfd_mach_arm_8,	  "cortex-a53"	    },
+  { bfd_mach_arm_8,	  "cortex-a55"	    },
+  { bfd_mach_arm_8,	  "cortex-a57"	    },
+  { bfd_mach_arm_8,	  "cortex-a72"	    },
+  { bfd_mach_arm_8,	  "cortex-a73"	    },
+  { bfd_mach_arm_8,	  "cortex-a75"	    },
+  { bfd_mach_arm_8,	  "cortex-a76"	    },
+  { bfd_mach_arm_6SM,	  "cortex-m0"	    },
+  { bfd_mach_arm_6SM,	  "cortex-m0plus"   },
+  { bfd_mach_arm_6SM,	  "cortex-m1"	    },
+  { bfd_mach_arm_8M_BASE, "cortex-m23"	    },
+  { bfd_mach_arm_7,	  "cortex-m3"	    },
+  { bfd_mach_arm_8M_MAIN, "cortex-m33"	    },
+  { bfd_mach_arm_7EM,	  "cortex-m4"	    },
+  { bfd_mach_arm_7EM,	  "cortex-m7"	    },
+  { bfd_mach_arm_7,	  "cortex-r4"	    },
+  { bfd_mach_arm_7,	  "cortex-r4f"	    },
+  { bfd_mach_arm_7,	  "cortex-r5"	    },
+  { bfd_mach_arm_8R,	  "cortex-r52"	    },
+  { bfd_mach_arm_7,	  "cortex-r7"	    },
+  { bfd_mach_arm_7,	  "cortex-r8"	    },
+  { bfd_mach_arm_4T,	  "ep9312"	    },
+  { bfd_mach_arm_8,	  "exynos-m1"	    },
+  { bfd_mach_arm_4,	  "fa526"	    },
+  { bfd_mach_arm_5TE,	  "fa606te"	    },
+  { bfd_mach_arm_5TE,	  "fa616te"	    },
+  { bfd_mach_arm_4,	  "fa626"	    },
+  { bfd_mach_arm_5TE,	  "fa626te"	    },
+  { bfd_mach_arm_5TE,	  "fa726te"	    },
+  { bfd_mach_arm_5TE,	  "fmp626"	    },
+  { bfd_mach_arm_XScale,  "i80200"	    },
+  { bfd_mach_arm_7,	  "marvell-pj4"	    },
+  { bfd_mach_arm_7,	  "marvell-whitney" },
+  { bfd_mach_arm_6K,	  "mpcore"	    },
+  { bfd_mach_arm_6K,	  "mpcorenovfp"	    },
+  { bfd_mach_arm_4,	  "sa1"		    },
+  { bfd_mach_arm_4,	  "strongarm"	    },
+  { bfd_mach_arm_4,	  "strongarm1"	    },
+  { bfd_mach_arm_4,	  "strongarm110"    },
+  { bfd_mach_arm_4,	  "strongarm1100"   },
+  { bfd_mach_arm_4,	  "strongarm1110"   },
+  { bfd_mach_arm_XScale,  "xscale"	    },
+  { bfd_mach_arm_8,	  "xgene1"	    },
+  { bfd_mach_arm_8,	  "xgene2"	    },
+  { bfd_mach_arm_ep9312,  "ep9312"	    },
+  { bfd_mach_arm_iWMMXt,  "iwmmxt"	    },
+  { bfd_mach_arm_iWMMXt2, "iwmmxt2"	    },
+  { bfd_mach_arm_unknown, "arm_any"	    }
 };
 
 static bfd_boolean
@@ -129,20 +222,34 @@ scan (const struct bfd_arch_info *info, const char *string)
 
 static const bfd_arch_info_type arch_info_struct[] =
 {
-  N (bfd_mach_arm_2,       "armv2",   FALSE, & arch_info_struct[1]),
-  N (bfd_mach_arm_2a,      "armv2a",  FALSE, & arch_info_struct[2]),
-  N (bfd_mach_arm_3,       "armv3",   FALSE, & arch_info_struct[3]),
-  N (bfd_mach_arm_3M,      "armv3m",  FALSE, & arch_info_struct[4]),
-  N (bfd_mach_arm_4,       "armv4",   FALSE, & arch_info_struct[5]),
-  N (bfd_mach_arm_4T,      "armv4t",  FALSE, & arch_info_struct[6]),
-  N (bfd_mach_arm_5,       "armv5",   FALSE, & arch_info_struct[7]),
-  N (bfd_mach_arm_5T,      "armv5t",  FALSE, & arch_info_struct[8]),
-  N (bfd_mach_arm_5TE,     "armv5te", FALSE, & arch_info_struct[9]),
-  N (bfd_mach_arm_XScale,  "xscale",  FALSE, & arch_info_struct[10]),
-  N (bfd_mach_arm_ep9312,  "ep9312",  FALSE, & arch_info_struct[11]),
-  N (bfd_mach_arm_iWMMXt,  "iwmmxt",  FALSE, & arch_info_struct[12]),
-  N (bfd_mach_arm_iWMMXt2, "iwmmxt2", FALSE, & arch_info_struct[13]),
-  N (bfd_mach_arm_unknown, "arm_any", FALSE, NULL)
+  N (bfd_mach_arm_2,         "armv2",          FALSE, & arch_info_struct[1]),
+  N (bfd_mach_arm_2a,        "armv2a",         FALSE, & arch_info_struct[2]),
+  N (bfd_mach_arm_3,         "armv3",          FALSE, & arch_info_struct[3]),
+  N (bfd_mach_arm_3M,        "armv3m",         FALSE, & arch_info_struct[4]),
+  N (bfd_mach_arm_4,         "armv4",          FALSE, & arch_info_struct[5]),
+  N (bfd_mach_arm_4T,        "armv4t",         FALSE, & arch_info_struct[6]),
+  N (bfd_mach_arm_5,         "armv5",          FALSE, & arch_info_struct[7]),
+  N (bfd_mach_arm_5T,        "armv5t",         FALSE, & arch_info_struct[8]),
+  N (bfd_mach_arm_5TE,       "armv5te",        FALSE, & arch_info_struct[9]),
+  N (bfd_mach_arm_XScale,    "xscale",         FALSE, & arch_info_struct[10]),
+  N (bfd_mach_arm_ep9312,    "ep9312",         FALSE, & arch_info_struct[11]),
+  N (bfd_mach_arm_iWMMXt,    "iwmmxt",         FALSE, & arch_info_struct[12]),
+  N (bfd_mach_arm_iWMMXt2,   "iwmmxt2",        FALSE, & arch_info_struct[13]),
+  N (bfd_mach_arm_5TEJ,      "armv5tej",       FALSE, & arch_info_struct[14]),
+  N (bfd_mach_arm_6,         "armv6",          FALSE, & arch_info_struct[15]),
+  N (bfd_mach_arm_6KZ,       "armv6kz",        FALSE, & arch_info_struct[16]),
+  N (bfd_mach_arm_6T2,       "armv6t2",        FALSE, & arch_info_struct[17]),
+  N (bfd_mach_arm_6K,        "armv6k",         FALSE, & arch_info_struct[18]),
+  N (bfd_mach_arm_7,         "armv7",          FALSE, & arch_info_struct[19]),
+  N (bfd_mach_arm_6M,        "armv6-m",        FALSE, & arch_info_struct[20]),
+  N (bfd_mach_arm_6SM,       "armv6s-m",       FALSE, & arch_info_struct[21]),
+  N (bfd_mach_arm_7EM,       "armv7e-m",       FALSE, & arch_info_struct[22]),
+  N (bfd_mach_arm_8,         "armv8-a",        FALSE, & arch_info_struct[23]),
+  N (bfd_mach_arm_8R,        "armv8-r",        FALSE, & arch_info_struct[24]),
+  N (bfd_mach_arm_8M_BASE,   "armv8-m.base",   FALSE, & arch_info_struct[25]),
+  N (bfd_mach_arm_8M_MAIN,   "armv8-m.main",   FALSE, & arch_info_struct[26]),
+  N (bfd_mach_arm_8_1M_MAIN, "armv8.1-m.main", FALSE, & arch_info_struct[27]),
+  N (bfd_mach_arm_unknown,   "arm_any",        FALSE, NULL)
 };
 
 const bfd_arch_info_type bfd_arm_arch =
@@ -190,7 +297,7 @@ bfd_arm_merge_machines (bfd *ibfd, bfd *obfd)
     {
       /* xgettext: c-format */
       _bfd_error_handler (_("\
-error: %B is compiled for the EP9312, whereas %B is compiled for XScale"),
+error: %pB is compiled for the EP9312, whereas %pB is compiled for XScale"),
 			  ibfd, obfd);
       bfd_set_error (bfd_error_wrong_format);
       return FALSE;
@@ -202,7 +309,7 @@ error: %B is compiled for the EP9312, whereas %B is compiled for XScale"),
     {
       /* xgettext: c-format */
       _bfd_error_handler (_("\
-error: %B is compiled for the EP9312, whereas %B is compiled for XScale"),
+error: %pB is compiled for the EP9312, whereas %pB is compiled for XScale"),
 			  obfd, ibfd);
       bfd_set_error (bfd_error_wrong_format);
       return FALSE;
@@ -304,7 +411,9 @@ bfd_arm_update_notes (bfd *abfd, const char *note_section)
   if (! arm_check_note (abfd, buffer, buffer_size, NOTE_ARCH_STRING, & arch_string))
     goto FAIL;
 
-  /* Check the architecture in the note against the architecture of the bfd.  */
+  /* Check the architecture in the note against the architecture of the bfd.
+     Newer architectures versions should not be added here as build attribute
+     are a better mechanism to convey ISA used.  */
   switch (bfd_get_mach (abfd))
     {
     default:
@@ -335,7 +444,7 @@ bfd_arm_update_notes (bfd *abfd, const char *note_section)
 	{
 	  _bfd_error_handler
 	    /* xgettext: c-format */
-	    (_("warning: unable to update contents of %s section in %B"),
+	    (_("warning: unable to update contents of %s section in %pB"),
 	     note_section, abfd);
 	  goto FAIL;
 	}
@@ -356,6 +465,9 @@ static struct
   const char * string;
   unsigned int mach;
 }
+
+/* Newer architectures versions should not be added here as build attribute are
+   a better mechanism to convey ISA used.  */
 architectures[] =
 {
   { "armv2",   bfd_mach_arm_2 },
